import xml.etree.ElementTree as ET
import json
import os

def id(obj, ns):
    anr = obj.find(".//ns:ANR", ns)
    trk = obj.find(".//ns:TRK", ns)
    if anr is not None:
        return f"{anr.text.strip()}#{trk.text.strip().zfill(3)}"
    else:
        print("Keine Archivnummer gefunden")
        return None

def traegertitel(obj, ns):
    ak_full = obj.find(".//ns:AK[@voll='j']", ns)
    if ak_full is not None:
        rhti = ak_full.find('ns:RHTI', ns)
        if rhti is not None and rhti.text:
            return rhti.text.strip()
        else:
            print("Kein RHTI Element in AK[@voll='j'] gefunden oder leerer Text")
            return None
    else:
        print("Kein AK Element mit voll='j' gefunden")
        return None
    
def tracktitel(obj, ns):
    ak = obj.find('ns:AK', ns)
    if ak is not None:
        rhti = ak.find('ns:RHTI', ns)
        if rhti is not None and rhti.text:
            return rhti.text.strip()
        else:
            print("Kein RHTI Element in AK gefunden oder leerer Text")
            return None
    else:
        print("Kein AK Element gefunden")
        return None

def konfkorpus(obj, ns):
    """
    Gibt das erste Element der Konfektionierung aus - dieses
    repräsentiert den Korpus
    """
    konf = obj.find('.//ns:KONF', ns)
    if konf is not None:
        return konf
    else: 
        print("kein Konfektionierungselement gefunden")
        return konf

def keywords(obj, ns, flat=False, sep="#"):
    """
    Gibt die Ergebnisse der mdh-Keyword-Extraction zurück
    """
    konf = konfkorpus(obj, ns)
    if konf is not None:
        # Keywords
        keywords = []
        for kwe in konf.findall('.//ns:KWE/ns:TEXT', ns):
            if kwe.text:
                keywords.append(kwe.text.strip())
                
        if flat:
            return f" {sep} ".join(keywords)
        else:
            return keywords
    
def transcript(obj, ns, add_speakers=False):
    """
    Gibt die Ergebnisse des MDH-Mining-Transkripts zurück
    """
    konf = konfkorpus(obj, ns)
    if konf is not None:
        transcript = ""
        for speaker in konf.findall('.//ns:TRANSCRIPT/ns:SPEAKER', ns):
            if speaker.text:
                if add_speakers:
                    transcript += f"SPEAKER {speaker.attrib['id']}: {speaker.text.strip()} "
                else:
                    transcript += speaker.text.strip() + " "
        return transcript.strip()
    
def personen(elem, ns):
    """
    Extrahiert Personenangaben aus dem Mitwirkenden- oder dem
    Urheber-Feld
    """
    person = {  
        'name': elem.find('.//ns:NAM', ns),
        'vorname': elem.find('.//ns:VNAM', ns),
        'anmerkung': elem.find('.//ns:ANM', ns),
        'funktion': elem.find('.//ns:FN', ns)
    }
    
    # über das Dictionary iterieren und die Texte extrahieren, sofern vorhanden
    return dict([(key, value.text if value is not None else None) for key, value in person.items()])


def mitwirkende(obj, ns):
    """
    Extrahier Personenangaben aus dem Mitwirkenden-Feld
    """
    results = []
    # nur das erste AK-Objekt zurückgeben (= Korpus)
    ak_korpus = obj.find('.//ns:AK', ns)
    bsg = ak_korpus.findall('.//ns:BSG', ns)
    if bsg is not None:
        for elem in bsg:
            results.append(personen(elem, ns))
        return results
    else:
        print("Keine Mitwirkenden gefunden")
        return None

def urheber(obj, ns):
    """
    Extrahier Personenangaben aus dem Urheber-Feld
    """
    results = []
    # nur das erste AK-Objekt zurückgeben (= Korpus)
    ak_korpus = obj.find('.//ns:AK', ns)
    bsg = ak_korpus.findall('.//ns:URH', ns)
    if bsg is not None:
        for elem in bsg:
            results.append(personen(elem, ns))
        return results
    else:
        print("Keine Mitwirkenden gefunden")
        return None    

def permalink(obj, ns):
    plugpara = obj.find('.//ns:PLUGPARA', ns)
    if plugpara is not None:
        instance = plugpara.get('instance')
        instance_num = instance[-1]
        konf = plugpara.get('konf')
        amo = plugpara.get('amo')
        ak = plugpara.get('ak')
        gk = plugpara.get('gk')
        
        link = f"https://linkresolver{instance_num}.ivz.cn.ard.de/linkresolver/resolve?context=hfdb{instance_num}&hfdb-recherche/permalink?typ=vollinfoAnzeige&vollinfoArt=vollinfokonf&ak={ak}&ko={konf}&amo={amo}&gkonf={gk}&destination={instance}"
        return link
    else:
        print("Fehler beim Auffinden des Plugpara-Elemente. Kein Permalink erstellt")
        return None

def parse_xml(xml_file, **kwargs):
    
    config = {
        'flat': False,
        'sep': "#",
        'add_speakers': False
    }
    
    config.update(kwargs)
    
    tree = ET.parse(xml_file)
    root = tree.getroot()
    
    # Namespace-Definition
    ns = {'ns': 'http://ard.de/sad/hfdb/vollinfo'}
    
    result = []

    for index, vi in enumerate(root.findall('ns:VI', ns)):
        
        # Laufende Nummerierung
        item = {'lfd': index + 1}
        
        # Archivnummer
        item['id'] = id(vi, ns)
        
        # Traegertitel
        item['traegertitel'] = traegertitel(vi, ns)
        
        # Tracktitel
        item['tracktitel'] = tracktitel(vi, ns)
        
        # Keywords
        item['keywords'] = keywords(vi, ns, flat=config['flat'], sep=config['sep'])
        
        # Transkript
        item['transcript'] = transcript(vi, ns, add_speakers=config['add_speakers'])
        
        # Mitwirkende
        item['mitwirkende'] = mitwirkende(vi, ns)
        
        # Urheber
        item['urheber'] = urheber(vi, ns)
        
        # Permalink
        item['permalink'] = permalink(vi, ns)
        
        result.append(item)
    
    return result

def xml_to_json(input_file, output_dir, **kwargs):
    data = parse_xml(input_file, **kwargs)
    output_file = os.path.join(output_dir, os.path.splitext(os.path.basename(input_file))[0] + '.json')
    
    with open(output_file, 'w', encoding='utf-8') as f:
        json.dump(data, f, ensure_ascii=False, indent=2)
    
    print(f"Die JSON-Datei wurde erstellt: {output_file}")
    return output_file
